"use strict";
/**
 * @athenna/core
 *
 * (c) João Lenon <lenon@athenna.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AthennaFactory = void 0;
const utils_1 = require("@secjs/utils");
const logger_1 = require("@athenna/logger");
const path_1 = require("path");
const config_1 = require("@athenna/config");
const getAppEnvironment_1 = require("../Utils/getAppEnvironment");
const ResolveClassExport_1 = require("../Utils/ResolveClassExport");
const AthennaErrorHandler_1 = require("../Utils/AthennaErrorHandler");
class AthennaFactory {
    constructor(fileName) {
        AthennaFactory.resolveNodeTs(fileName);
        process.env.NODE_ENV = getAppEnvironment_1.getAppEnvironment(utils_1.Path.config(`app${AthennaFactory.extension}`));
        config_1.resolveEnvFile();
        Config.load(utils_1.Path.config());
        AthennaFactory.logger = new logger_1.Logger().channel('application', {
            formatterConfig: {
                context: AthennaFactory.name,
            },
        });
        if (!Env('APP_DEBUG') &&
            (Env('NODE_ENV') === 'test' || Env('NODE_ENV') === 'testing')) {
            AthennaFactory.logger = {
                log: () => { },
                error: () => { },
            };
        }
        else {
            console.clear();
        }
        const providers = AthennaFactory.getProviders();
        AthennaFactory.registerProviders(providers);
        AthennaFactory.bootProviders(providers);
        AthennaFactory.preloadFiles();
    }
    static getProviders() {
        const providers = Config.get('app.providers');
        const providersNormalized = [];
        providers.forEach(Provider => {
            providersNormalized.push(ResolveClassExport_1.ResolveClassExport.resolve(Provider));
        });
        providersNormalized.forEach(Provider => AthennaFactory.logger.log(`Registering ${Provider.name}`));
        return providersNormalized;
    }
    static bootProviders(providers) {
        providers.forEach(Provider => {
            const provider = new Provider();
            provider.registerAttributes();
            if (provider.boot)
                provider.boot();
        });
    }
    static registerProviders(providers) {
        providers.forEach(Provider => {
            const provider = new Provider();
            provider.registerAttributes();
            if (provider.register)
                provider.register();
        });
    }
    static preloadFiles() {
        const preloads = Config.get('app.preloads');
        preloads.forEach(preload => {
            preload = path_1.normalize(preload);
            const { dir, name } = path_1.parse(utils_1.Path.config(preload));
            AthennaFactory.logger.log(`Preloading ${name} file`);
            require(`${dir}/${name}${this.extension}`);
        });
    }
    static resolveNodeTs(fileName) {
        const { ext } = path_1.parse(fileName);
        if (ext === '.ts')
            process.env.NODE_TS = 'true';
        else
            process.env.NODE_TS = 'false';
        AthennaFactory.extension = ext;
    }
    async http() {
        const http = ioc.safeUse('Athenna/Core/HttpServer');
        const route = ioc.safeUse('Athenna/Core/HttpRoute');
        http.setErrorHandler(AthennaErrorHandler_1.AthennaErrorHandler.http);
        route.register();
        const port = Config.get('app.port');
        const host = Config.get('app.host');
        await http.listen(port, host);
        AthennaFactory.logger.log(`Http server started on http://${host}:${port}`);
        return http;
    }
}
exports.AthennaFactory = AthennaFactory;
